.286
.MODEL SMALL,OS_DOS

;
;   GESTION DU TEMPS SYSTEME
;
;   Traitement de l'interruption de type 08 = horloge temps-rel du systme
;   Dfinition du processus HORLOGE de traitement du temps
;
; HORLOGE.ASM
;
;  ART OF CODE 1994
;
;

EXTERNDEF AFFDEC : NEAR

INCLUDE dos.inc
INCLUDE types.inc
INCLUDE const.inc
INCLUDE etats.inc
INCLUDE descript.inc
INCLUDE semphor.inc
INCLUDE process.inc
INCLUDE gestlist.inc
INCLUDE donnees.inc
INCLUDE code.inc

; Pile prive du processus HORLOGE

PileHorloge SEGMENT WORD 'STACK'    ; pile prive du processus
    WORD (DimPile-NbRegs) DUP (?)

SP_PileHorloge LABEL WORD           ; sommet actuel de la pile prive
    WORD (NbRegs - 2)     DUP (?)   ; valeur initiale des registres du p

    WORD ETAT_INITIAL               ; valeur initiale du mot d'tat
    WORD OFFSET horloge             ; point d'entre
PileHorloge ENDS

; Segment de donnes du BIOS

donnees_bios SEGMENT AT 0040h
    org 003Fh
    MOTOR_STATUS    BYTE ?
    MOTOR_COUNT     BYTE ?
    org 006Ch
    TIMER_LOW       WORD ?
    TIMER_HIGH      WORD ?
    TIMER_OFL       BYTE ?
donnees_bios ENDS

.DATA

pHorloge    PROCESSUS   <{,,,TypeProcessus,OCCUPE,0},OFFSET SP_PileHorloge,PileHorloge,NIL>

SemHorloge  SEMAPHORE	<>  ; smaphore de synchro sur int timer

EXTERNDEF sHorloge : SEMAPHORE

EX_INT08h   DWORD   ?       ; sauvegarde vecteur int timer DOS

INT08h      DWORD   timer   ; pointeur service int timer

.CODE

;               Traitement de l'interruption du timer
;
; Cette procdure est dclenche par l'interruption de type 8.
; Elle dbloque la tache HORLOGE apres avoir acquitt l'interruption au controleur.
;

timer PROC FAR
    push ds
    push ax
    mov  ax,@data
    mov  ds,ax
ASSUME ds:@data
    STI                 ; dmasquage effectif qu'apres l'instruction suivante
    inc [NonPreemptif]  ; le noyau n'est plus preemptif

;   ---------------- Acquitte l'interruption au controleur -------------------

    mov al,EOI           ; (al) := octet d'acquittement
    out PortCtrlpic,al   ; .... Envoy au PIC 8259A

;  Dbloque le processus Horloge

    push si
    lea si,SemHorloge   ; (si) ---> smaphore de synchronisation
    call V              ; dbloque le processus
    pop si

; Termine et force la commutation

    dec [NonPreemptif]
    pop ax
    call dispatcher
    pop ds
ASSUME ds:NOTHING
    IRET
timer ENDP

; Code de la tache d'horloge

TempsDos PROC
	pushf
	push ax
	ASSUME ds:donnees_bios
	inc [TIMER_LOW]     ; actualise l'heure
	jnz TD1             ; dpassement ?
	inc [TIMER_HIGH]    ; oui ! ajuste les poids forts
TD1:
	cmp [TIMER_HIGH],24 ; 24 heures coules ?
	jnz TD2
	cmp [TIMER_LOW],0B0h
	jnz TD2

	xor ax,ax           ; change le jour
	mov [TIMER_LOW],ax
	mov [TIMER_HIGH],ax
	mov [TIMER_OFL],1
TD2:
	pop ax
	popf
	RET
TempsDos ENDP

MoteurDsk   PROC
	    pushf
	    push ax
	    push dx
	    ASSUME ds:donnees_bios
	    dec [MOTOR_COUNT]
	    jnz FinMtDSK
	    and [MOTOR_STATUS],0F0h
	    mov al,0Ch
	    mov dx,PortCTRLfdc
	    out dx,al
FinMtDSK:
	    pop dx
	    pop ax
	    popf
	    RET
MoteurDsk   ENDP



Horloge PROC
	ASSUME DS:@data
	mov ax,@data
	mov ds,ax

	; initialisation du smaphore SemHorloge

	lea si,SemHorloge       ; (si) ---> smaphore de synchronisation
	mov (SEMAPHORE PTR [si]).Compteur,0 ; smaphore : val = 0
	call InitVide           ; On initialise les pointeurs de la file

	; initialisation du vecteur d'interruption

	cld

	xor ax,ax	    ; On sauvegarde l'ancien vecteur
	mov ds,ax	    ; d'interruption 08h
	mov ax,@data
	mov es,ax
	mov si,20h
	lea di,Ex_Int08h
	movsw
	movsw

	mov ax,@data	    ; On place le nouveau vecteur
	mov ds,ax
	xor ax,ax
	mov es,ax
	mov di,20h
	lea si,Int08h
	pushf
	cli		    ; pas d'interruptions durant le transfert
	movsw
	movsw
	popf		    ; retablissement des interruptions si il y a lieu

repeter:
	lea si,SemHorloge           ; (si) ---> smaphore de synchronisation
	Call P			    ; WAIT (SemHorloge)
	mov ax,donnees_bios
	mov ds,ax
	Call TempsDos		    ; on met  jour l'horloge
	Call MoteurDsk		    ; on arrete le moteur de disquette
	mov ax,@data
	mov ds,ax
	lea si,sHorloge 	    ; on active le processus d'affichage de
	call v			    ; l'horloge  l'cran
	mov si,offset FinNoyau	    ; on teste si on a fini
	cmp BYTE PTR [si],FIN
	Jne repeter		    ; non, on continue

	mov di,20h		    ; on rtablit l'ancien vecteur d'interruption
	lea si,Ex_Int08h
	pushf
	cli			    ; pas d'interruptions pendant ce traitement
	movsw
	movsw
	popf			    ; interruptions rtablies


	mov si,offset NoyauFini
	mov BYTE PTR [si],FIN	    ; on signale  pNoyau que c'est fait !

	mov si,[EnCours]
	mov (DESCRIPTEUR PTR [si]).Etat,TERMINE

	call DISPATCHER		    ; On termine le processus Horloge
				    ; Le dipatcher commutera sur pNoyau
				    ; qui terminera proprement.
	; il ne faut surtout pas terminer par un 'RET'
	; sinon, plantage !!! ( La pile ne contient pas d'adresse de retour ).

Horloge ENDP

END
